@@ -1,8 +1,11 @@
 from pathlib import Path
+from typing import Union
 
 import attr
+import click
 from _pytask.config import hookimpl
 from _pytask.console import console
+from _pytask.shared import get_first_non_none_value
 from _pytask.shared import reduce_node_name
 from rich.live import Live
 from rich.status import Status
@@ -10,13 +13,59 @@ from rich.table import Table
 from rich.text import Text
 
 
+@hookimpl
+def pytask_extend_command_line_interface(cli):
+    """Extend command line interface."""
+    additional_parameters = [
+        click.Option(
+            ["--n-entries-in-table"],
+            default=None,
+            help="How many entries to display in the table during the execution. "
+            "Tasks which are running are always displayed.  [default: 15]",
+        ),
+    ]
+    cli.commands["build"].params.extend(additional_parameters)
+
+
+@hookimpl
+def pytask_parse_config(config, config_from_cli, config_from_file):
+    config["n_entries_in_table"] = get_first_non_none_value(
+        config_from_cli,
+        config_from_file,
+        key="n_entries_in_table",
+        default=15,
+        callback=_parse_n_entries_in_table,
+    )
+
+
+def _parse_n_entries_in_table(value: Union[int, str, None]) -> int:
+    if value in ["none", "None", None, ""]:
+        out = None
+    elif isinstance(value, int) and value >= 1:
+        out = value
+    elif isinstance(value, str) and value.isdigit() and int(value) >= 1:
+        out = int(value)
+    elif value == "all":
+        out = 1_000_000
+    else:
+        raise ValueError(
+            "'n_entries_in_table' can either be 'None' or an integer bigger than one."
+        )
+    return out
+
+
 @hookimpl
 def pytask_post_parse(config):
     live_manager = LiveManager()
     config["pm"].register(live_manager, "live_manager")
 
-    if config["verbose"] >= 0:
-        live_execution = LiveExecution(live_manager, config["paths"])
+    if config["verbose"] >= 1:
+        live_execution = LiveExecution(
+            live_manager,
+            config["paths"],
+            config["n_entries_in_table"],
+            config["verbose"],
+        )
         config["pm"].register(live_execution)
 
     live_collection = LiveCollection(live_manager)
@@ -33,7 +82,7 @@ class LiveManager:
 
     """
 
-    _live = Live(renderable=None, console=console, auto_refresh=False)
+    _live = Live(renderable=None, console=console, auto_refresh=True)
 
     def start(self):
         self._live.start()
@@ -56,9 +105,6 @@ class LiveManager:
     def update(self, *args, **kwargs):
         self._live.update(*args, **kwargs)
 
-    def refresh(self):
-        self._live.refresh()
-
     @property
     def is_started(self):
         return self._live.is_started
@@ -69,6 +115,8 @@ class LiveExecution:
 
     _live_manager = attr.ib(type=LiveManager)
     _paths = attr.ib(type=Path)
+    _n_entries_in_table = attr.ib(type=int)
+    _verbose = attr.ib(type=int)
     _running_tasks = attr.ib(factory=set)
     _reports = attr.ib(factory=list)
 
@@ -76,6 +124,7 @@ class LiveExecution:
     def pytask_execute_build(self):
         self._live_manager.start()
         yield
+        self._update_table(reduce_table=False)
         self._live_manager.stop(transient=False)
 
     @hookimpl(tryfirst=True)
@@ -83,18 +132,38 @@ class LiveExecution:
         self.update_running_tasks(task)
         return True
 
-    @hookimpl(tryfirst=True)
+    @hookimpl
     def pytask_execute_task_log_end(self, report):
         self.update_reports(report)
         return True
 
-    def _generate_table(self):
+    def _generate_table(self, reduce_table: bool) -> Union[None, Table]:
+        """Generate the table.
+
+        First, display all completed tasks and, then, all running tasks.
+
+        The number of entries can be limited. All running tasks are always displayed and
+        if more entries are requested, the list is filled up with completed tasks.
+
+        """
         if self._running_tasks or self._reports:
+
+            n_reports_to_display = self._n_entries_in_table - len(self._running_tasks)
+            if not reduce_table:
+                relevant_reports = self._reports
+            elif n_reports_to_display >= 1:
+                relevant_reports = self._reports[-n_reports_to_display:]
+            else:
+                relevant_reports = []
+
             table = Table("Task", "Outcome")
-            for report in self._reports:
-                table.add_row(
-                    report["name"], Text(report["symbol"], style=report["color"])
-                )
+            for report in relevant_reports:
+                if report["symbol"] in ("s", "p") and self._verbose < 2:
+                    pass
+                else:
+                    table.add_row(
+                        report["name"], Text(report["symbol"], style=report["color"])
+                    )
             for running_task in self._running_tasks:
                 table.add_row(running_task, "running")
         else:
@@ -102,10 +171,9 @@ class LiveExecution:
 
         return table
 
-    def _update_table(self):
-        table = self._generate_table()
+    def _update_table(self, reduce_table: bool = True):
+        table = self._generate_table(reduce_table)
         self._live_manager.update(table)
-        self._live_manager.refresh()
 
     def update_running_tasks(self, new_running_task):
         reduced_task_name = reduce_node_name(new_running_task, self._paths)
@@ -160,7 +228,6 @@ class LiveCollection:
     def _update_status(self):
         status = self._generate_status()
         self._live_manager.update(status)
-        self._live_manager.refresh()
 
     def _generate_status(self):
         msg = f"Collected {self._n_collected_tasks} tasks."
